package gov.cms.fiss.pricers.ltch.core.rules;

import static gov.cms.fiss.pricers.ltch.core.codes.PaymentType.STANDARD_OLD;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.ltch.api.BillingRecord;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import gov.cms.fiss.pricers.ltch.core.codes.ErrorCode;
import gov.cms.fiss.pricers.ltch.core.codes.PaymentType;
import java.math.BigDecimal;
import java.math.RoundingMode;

public class CalcFinalPayment implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {

  /**
   *
   *
   * <pre>
   * ***************************************************************
   * *   CALCULATE THE "FINAL" PAYMENT AMOUNT.                     *
   * *   UNIQUE CALCULATION FOR EACH CLAIM PAYMENT TYPE COMBO      *
   * ***************************************************************
   * </pre>
   *
   * Converted from {@code 8000-CALC-FINAL-PMT}
   */
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    final PaymentType paymentType = calculationContext.getPaymentType();
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();
    final BillingRecord billingRecord = calculationContext.getBillingRecord();
    if (!ErrorCode.isErrorCode(prospectivePaymentRecord.getRtc())) {
      switch (paymentType) {
          // *-------------------------------------------------------------*
          // * OLD POLICY CLAIMS (100% STANDARD PAYMENT)                   *
          // *-------------------------------------------------------------*
        case STANDARD_OLD:
          // COMPUTE PPS-FINAL-PAY-AMT =
          //         PPS-DRG-ADJ-PAY-AMT + PPS-OUTLIER-PAY-AMT
          prospectivePaymentRecord.setFinalPaymentAmount(
              prospectivePaymentRecord
                  .getAdjustedPaymentAmount()
                  .add(prospectivePaymentRecord.getOutlierPaymentAmount())
                  .setScale(2, RoundingMode.HALF_UP));
          break;
          // NEW POLICY CLAIMS (ANY PAYMENT TYPE) ONLY APPLICABLE PAYMENT FIELDS CONTAIN VALUES > $0
          // APPLY BUDGET NEUTRALITY AND/OR BLEND TO PAYMENT IF NEEDED ANY APPLICABLE BUDGET
          // NEUTRALITY AND/OR BLEND WAS ALREADY APPLIED TO OUTLIER PAYMENT APPLY BUDGET NEUTRALITY
          // AND SITE-NEUTRAL IPPS ADJ. TO 100% SITE-NEUTRAL PAYMENTS
        case SITE_NEUTRAL:
          // COMPUTE PPS-SITE-NEUTRAL-COST-PMT ROUNDED =
          //        PPS-SITE-NEUTRAL-COST-PMT *
          //        H-BDGT-NEUT-FACTOR
          prospectivePaymentRecord
              .getPaymentData()
              .setSiteNeutralCostPayment(
                  prospectivePaymentRecord
                      .getPaymentData()
                      .getSiteNeutralCostPayment()
                      .multiply(calculationContext.getHoldBudgetNeutralFactor())
                      .setScale(2, RoundingMode.HALF_UP));
          // COMPUTE PPS-SITE-NEUTRAL-IPPS-PMT ROUNDED =
          //         PPS-SITE-NEUTRAL-IPPS-PMT *
          //         H-BDGT-NEUT-FACTOR *
          //         H-SITE-NEUTRAL-IPPS-ADJ
          prospectivePaymentRecord
              .getPaymentData()
              .setSiteNeutralIppsPayment(
                  prospectivePaymentRecord
                      .getPaymentData()
                      .getSiteNeutralIppsPayment()
                      .multiply(calculationContext.getHoldBudgetNeutralFactor())
                      .multiply(calculationContext.getHoldSiteNeutralIppsAdj())
                      .setScale(2, RoundingMode.HALF_UP));
          // COMPUTE PPS-OUTLIER-PAY-AMT ROUNDED =
          //         PPS-OUTLIER-PAY-AMT *
          //         H-SITE-NEUTRAL-IPPS-ADJ
          prospectivePaymentRecord.setOutlierPaymentAmount(
              prospectivePaymentRecord
                  .getOutlierPaymentAmount()
                  .multiply(calculationContext.getHoldSiteNeutralIppsAdj())
                  .setScale(2, RoundingMode.HALF_UP));
          break;
          // *-------------------------------------------------------*
          // * APPLY BLEND PERCENTS, BUDGET NEUT, AND SITE-NEUTRAL   *
          // * IPPS ADJUSTMENT TO BLENDED PAYMENTS                   *
          // *-------------------------------------------------------*
        case BLEND:
          // COMPUTE PPS-SITE-NEUTRAL-COST-PMT ROUNDED =
          //         PPS-SITE-NEUTRAL-COST-PMT *
          //         H-BDGT-NEUT-FACTOR *
          //         H-BLEND-SNT
          prospectivePaymentRecord
              .getPaymentData()
              .setSiteNeutralCostPayment(
                  prospectivePaymentRecord
                      .getPaymentData()
                      .getSiteNeutralCostPayment()
                      .multiply(calculationContext.getHoldBudgetNeutralFactor())
                      .multiply(calculationContext.getHoldBlendSiteNeutral())
                      .setScale(2, RoundingMode.HALF_UP));
          // COMPUTE PPS-SITE-NEUTRAL-IPPS-PMT ROUNDED =
          //         PPS-SITE-NEUTRAL-IPPS-PMT *
          //         H-BDGT-NEUT-FACTOR *
          //         H-SITE-NEUTRAL-IPPS-ADJ *
          //         H-BLEND-SNT
          prospectivePaymentRecord
              .getPaymentData()
              .setSiteNeutralIppsPayment(
                  prospectivePaymentRecord
                      .getPaymentData()
                      .getSiteNeutralIppsPayment()
                      .multiply(calculationContext.getHoldBudgetNeutralFactor())
                      .multiply(calculationContext.getHoldSiteNeutralIppsAdj())
                      .multiply(calculationContext.getHoldBlendSiteNeutral())
                      .setScale(2, RoundingMode.HALF_UP));
          // COMPUTE PPS-STANDARD-FULL-PMT ROUNDED =
          //         PPS-STANDARD-FULL-PMT * H-BLEND-STD
          prospectivePaymentRecord
              .getPaymentData()
              .setStandardFullPayment(
                  prospectivePaymentRecord
                      .getPaymentData()
                      .getStandardFullPayment()
                      .multiply(calculationContext.getHoldBlendStandard())
                      .setScale(2, RoundingMode.HALF_UP));
          // COMPUTE PPS-STANDARD-SSO-PMT ROUNDED =
          //         PPS-STANDARD-SSO-PMT * H-BLEND-STD
          prospectivePaymentRecord
              .getPaymentData()
              .setStandardShortStayOutliersPayment(
                  prospectivePaymentRecord
                      .getPaymentData()
                      .getStandardShortStayOutliersPayment()
                      .multiply(calculationContext.getHoldBlendStandard())
                      .setScale(2, RoundingMode.HALF_UP));
          break;
        default:
          break;
      }
      // *-------------------------------------------------------*
      // * SUM PAYMENT FIELDS FOR FINAL PAYMENT                  *
      // *-------------------------------------------------------*
      if (!paymentType.equals(STANDARD_OLD)) {
        // COMPUTE PPS-FINAL-PAY-AMT =
        //         PPS-STANDARD-FULL-PMT +
        //         PPS-STANDARD-SSO-PMT +
        //         PPS-SITE-NEUTRAL-COST-PMT +
        //         PPS-SITE-NEUTRAL-IPPS-PMT +
        //         PPS-OUTLIER-PAY-AMT
        prospectivePaymentRecord.setFinalPaymentAmount(
            prospectivePaymentRecord
                .getPaymentData()
                .getStandardFullPayment()
                .add(
                    prospectivePaymentRecord.getPaymentData().getStandardShortStayOutliersPayment())
                .add(prospectivePaymentRecord.getPaymentData().getSiteNeutralCostPayment())
                .add(prospectivePaymentRecord.getPaymentData().getSiteNeutralIppsPayment())
                .add(prospectivePaymentRecord.getOutlierPaymentAmount())
                .setScale(2, RoundingMode.HALF_UP));
        // *-------------------------------------------------------------*
        // * IF THERE'S AN LTCH DPP PAYMENT ADJUSTMENT THEN              *
        // * CALCULATE THE 'LTCH DPP ADJUSTMENT AMOUNT' AND              *
        // * ADD IT TO THE FINAL PAYMENT                                 *
        // *-------------------------------------------------------------*
        if ("Y".equals(billingRecord.getDischargePaymentPercentageIndicator())) {
          calculateIPPSLikeAmt(calculationContext);
          // COMPUTE PPS-LTCH-DPP-ADJ-AMT =
          //    H-IPPS-LIKE-AMT - H-PRE-DPP-PAY
          prospectivePaymentRecord.setDischargePaymentPercentageAmount(
              calculationContext
                  .getHoldIPPSLikeAmt()
                  .subtract(prospectivePaymentRecord.getFinalPaymentAmount())
                  .setScale(2, RoundingMode.HALF_UP));
          // COMPUTE PPS-FINAL-PAY-AMT =
          //    H-PRE-DPP-PAY + PPS-LTCH-DPP-ADJ-AMT
          prospectivePaymentRecord.setFinalPaymentAmount(
              prospectivePaymentRecord
                  .getFinalPaymentAmount()
                  .add(prospectivePaymentRecord.getDischargePaymentPercentageAmount()));
        }
      }
    }
  }

  void calculateIPPSLikeAmt(LtchPricerContext calculationContext) {
    // *-------------------------------------------------------------*
    // * GET AN 'IPPS-LIKE AMOUNT' TO BE USED FOR THE DPP ADJUSTMENT *
    // *-------------------------------------------------------------*
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();
    // *-------------------------------------------------------------*
    // * GET THE IPPS PAY AMT                                        *
    // *-------------------------------------------------------------*
    // TODO don't do this
    new CalculateIppsComparablePayment().priceClaim(calculationContext);
    // *-------------------------------------------------------------*
    // * GET THE IPPS PMT HIGH COST OUTLIER (HCO)                    *
    // *-------------------------------------------------------------*
    // COMPUTE PPS-OUTLIER-THRESHOLD ROUNDED =
    //             H-IPPS-PAY-AMT + H-FIXED-LOSS-AMT-SNT
    prospectivePaymentRecord.setOutlierThreshold(
        calculationContext
            .getHoldIppsPayAmount()
            .add(calculationContext.getHoldFixedLossAmountSiteNeutral()));
    // IF PPS-FAC-COSTS > PPS-OUTLIER-THRESHOLD
    if (BigDecimalUtils.isGreaterThan(
        prospectivePaymentRecord.getFacilityCosts(),
        prospectivePaymentRecord.getOutlierThreshold())) {
      // COMPUTE H-IPPS-LIKE-AMT-OUTLIER =
      //        (PPS-FAC-COSTS - PPS-OUTLIER-THRESHOLD) * .8
      calculationContext.setHoldIPPSOutlier(
          prospectivePaymentRecord
              .getFacilityCosts()
              .subtract(prospectivePaymentRecord.getOutlierThreshold())
              .multiply(new BigDecimal("0.8")));
    }
    // *-------------------------------------------------------------*
    // * COMPUTE THE IPPS LIKE AMOUNT                                *
    // *-------------------------------------------------------------*
    // COMPUTE H-IPPS-LIKE-AMT =
    //             H-IPPS-PAY-AMT +
    //             H-IPPS-LIKE-AMT-OUTLIER.
    calculationContext.setHoldIPPSLikeAmt(
        calculationContext.getHoldIppsPayAmount().add(calculationContext.getHoldIPPSOutlier()));
  }
}
