package gov.cms.fiss.pricers.ltch.core.rules;

import ch.obermuhlner.math.big.BigDecimalMath;
import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.api.ProviderRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import gov.cms.fiss.pricers.ltch.core.codes.CbsaProviderType;
import gov.cms.fiss.pricers.ltch.core.codes.ErrorCode;
import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;

public class CalculateIppsComparablePayment
    implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {

  private static final String RURAL_BEDS = "500";
  private static final String URBAN_BEDS = "100";

  /**
   *
   *
   * <pre>
   * ***************************************************************
   * *   CALCULATE THE IPPS COMPARABLE PAYMENT COMPONENTS AND      *
   * *   PER DIEM PAYMENT AMOUNT                                   *
   * ***************************************************************
   * </pre>
   *
   * Converted from {@code 3650-SS-IPPS-COMP-PMT}
   */
  // TODO split up by steps
  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();
    if (calculationContext.isPaymentBlendOrSiteNeutral()
        && !ErrorCode.isErrorCode(prospectivePaymentRecord.getRtc())) {
      calculationContext.setIppsComparablePayment(true);
    }
    if (calculationContext.getIppsComparablePayment()) {
      calculationContext.setIppsComparablePayment(false);
      ProviderRecord providerRecord = calculationContext.getProviderRecord();
      // *** -------------------------------------------------------
      // *** OPERATING TEACHING ADJUSTMENT
      // *** -------------------------------------------------------
      // COMPUTE H-OPER-IME-TEACH ROUNDED =
      //        1.35 * ((1 + H-INTERN-RATIO) ** .405 - 1).
      BigDecimal internRatio =
          BigDecimalMath.pow(
              calculationContext.getHoldInternRatio().add(BigDecimal.ONE),
              new BigDecimal("0.405"),
              new MathContext(10));
      BigDecimal operatingTeachingAdjustment =
          new BigDecimal("1.35")
              .multiply(internRatio.subtract(BigDecimal.ONE))
              .setScale(9, RoundingMode.HALF_UP);

      // *** -------------------------------------------------------
      // *** CAPITAL TEACHING ADJUSTMENT (2.7183 = E ROUNDED)
      // *** STARTING FY 2009 - REDUCE H-CAPI-IME-TEACH ROUNDED 50%
      // *** 02/17/2009 - 50% REDUCTION REMOVED DUE TO STIMULUS BILL
      // ***              THIS CHANGE IS RETROACTIVE TO 10/01/2008
      // *** -------------------------------------------------------
      // IF H-CAPI-IME-RATIO > 1.5000
      //   MOVE 1.5000 TO H-CAPI-IME-RATIO.
      if (BigDecimalUtils.isGreaterThan(
          calculationContext.getHoldCapitalTeachingAdjustmentRatio(), new BigDecimal("1.5"))) {
        calculationContext.setHoldCapitalTeachingAdjustmentRatio(
            new BigDecimal("1.5").setScale(4, RoundingMode.HALF_UP));
      }

      // COMPUTE H-CAPI-IME-TEACH ROUNDED =
      //        ((2.7183 ** (.2822 * H-CAPI-IME-RATIO)) - 1).
      calculationContext.setHoldCapitalImeTeach(
          BigDecimalMath.pow(
                  new BigDecimal("2.7183"),
                  new BigDecimal("0.2822")
                      .multiply(calculationContext.getHoldCapitalTeachingAdjustmentRatio()),
                  new MathContext(10))
              .subtract(BigDecimal.ONE)
              .setScale(9, RoundingMode.HALF_UP));

      // *** -------------------------------------------------------
      // *** OPERATING DSH ADJUSTMENT
      // *** -------------------------------------------------------
      // *1) DETERMINE WHETHER THE PROVIDER IS URBAN OR RURAL
      // *---------------------------------------------------
      CbsaProviderType cbsaProviderType = getCbsaProviderType(calculationContext);

      // *2) CALCULATE THE OPERATING DSH PERCENT
      // *--------------------------------------
      // COMPUTE H-OPER-DSH-PCT ROUNDED =
      //         P-NEW-SSI-RATIO + P-NEW-MEDICAID-RATIO.
      calculationContext.setHoldOperatingDshPercent(
          providerRecord
              .getSsiRatio()
              .add(providerRecord.getMedicaidRatio())
              .setScale(4, RoundingMode.HALF_UP));

      // *3) DETERMINE THE PROVIDER'S GEOGRAPHIC CLASSIFICATION
      // *-----------------------------------------------------
      // *4) CALCULATE OPERATING DSH AMOUNT BASED ON GEOGRAPHIC CLASS
      // *-----------------------------------------------------------
      calculateOperatingDshAmount(calculationContext, cbsaProviderType);

      // *** -------------------------------------------------------
      // *** CAPITAL DSH ADJUSTMENT (2.7183 = E ROUNDED)
      // *** -------------------------------------------------------
      // IF URBAN-CBSA AND H-BED-SIZE >= 100
      if (cbsaProviderType.equals(CbsaProviderType.URBAN)
          && BigDecimalUtils.isGreaterThanOrEqualTo(
              calculationContext.getHoldBedSize(), new BigDecimal("100"))) {
        // COMPUTE H-CAPI-DSH ROUNDED =
        //          2.7183 ** (.2025 * H-OPER-DSH-PCT) - 1
        calculationContext.setHoldCapitalDsh(
            BigDecimalMath.pow(
                    new BigDecimal("2.7183"),
                    new BigDecimal("0.2025")
                        .multiply(calculationContext.getHoldOperatingDshPercent()),
                    new MathContext(10))
                .subtract(BigDecimal.ONE)
                .setScale(4, RoundingMode.HALF_UP));
      } else {
        calculationContext.setHoldCapitalDsh(BigDecimal.ZERO.setScale(4, RoundingMode.HALF_UP));
      }

      // *** -------------------------------------------------------
      // *** OPERATING PAYMENT (STANDARD AMOUNT)
      // *** -------------------------------------------------------
      BigDecimal operatingCola;
      if ("02".equals(providerRecord.getNewStateCode())
          || "12".equals(providerRecord.getNewStateCode())) {
        operatingCola = providerRecord.getCostOfLivingAdjustment();
      } else {
        operatingCola = BigDecimal.ONE;
      }

      // COMPUTE H-STAND-AMT-OPER-PMT ROUNDED =
      //       ( (H-IPPS-NAT-LABOR-SHR * H-IPPS-WAGE-INDEX) +
      //         (H-IPPS-NAT-NONLABOR-SHR * H-OPER-COLA) ) *
      //         H-IPPS-DRG-WGT * (1 + H-OPER-IME-TEACH + H-OPER-DSH ).
      BigDecimal teachDsh =
          BigDecimal.ONE
              .add(operatingTeachingAdjustment)
              .add(calculationContext.getHoldOperatingDshAmount());

      calculationContext.setHoldStandardAmountOperatingPayment(
          calculationContext
              .getHoldIppsNationalLaborShare()
              .multiply(calculationContext.getHoldProvIppsCBSA().getWageIndex())
              .add(calculationContext.getHoldIppsNationalNonLaborShare().multiply(operatingCola))
              .multiply(calculationContext.getHoldIppsDrgWeight())
              .multiply(teachDsh)
              .setScale(2, RoundingMode.HALF_UP));

      // *** -------------------------------------------------------
      // *** CAPITAL PAYMENT (CAPITAL RATE)
      // *** -------------------------------------------------------
      // COMPUTE H-CAPI-COLA ROUNDED =
      //       (.3152 * (H-OPER-COLA - 1) + 1).
      calculationContext.setHoldCapitalCola(
          new BigDecimal("0.3152")
              .multiply(operatingCola.subtract(BigDecimal.ONE))
              .add(BigDecimal.ONE)
              .setScale(3, RoundingMode.HALF_UP));

      // *--------------------------------------------------------------*
      // *   LARGE-URBAN ADD-ON ELIMINATED FOR VERSIONS 2008.1 &        *
      // *   LATER (CHANGED FROM 1.03 TO 1.00)                          *
      // *--------------------------------------------------------------*
      calculationContext.setHoldLargeUrbanAddon(BigDecimal.ONE);

      // COMPUTE H-CAPI-GAF ROUNDED =
      //       (H-IPPS-WAGE-INDEX ** .6848).
      calculationContext.setHoldCapitalGeographicAreaFactor(
          BigDecimalMath.pow(
                  calculationContext.getHoldProvIppsCBSA().getWageIndex(),
                  new BigDecimal("0.6848"),
                  new MathContext(10))
              .setScale(4, RoundingMode.HALF_UP));

      // COMPUTE H-CAPI-PMT ROUNDED =
      //       H-IPPS-CAPI-STD-FED-RATE * H-IPPS-DRG-WGT * H-CAPI-GAF *
      //       H-LRGURB-ADD-ON *  H-CAPI-COLA *
      //       (1 + H-CAPI-IME-TEACH + H-CAPI-DSH).
      calculationContext.setHoldCapitalPayment(
          calculationContext
              .getHoldIppsCapitalStandardFedRate()
              .multiply(calculationContext.getHoldIppsDrgWeight())
              .multiply(calculationContext.getHoldCapitalGeographicAreaFactor())
              .multiply(calculationContext.getHoldLargeUrbanAddon())
              .multiply(calculationContext.getHoldCapitalCola())
              .multiply(
                  BigDecimal.ONE
                      .add(calculationContext.getHoldCapitalImeTeach())
                      .add(calculationContext.getHoldCapitalDsh()))
              .setScale(2, RoundingMode.HALF_UP));

      // *** -------------------------------------------------------
      // *** IPPS COMPARABLE TOTAL PAYMENT (OPERATING + CAPITAL)
      // *** -------------------------------------------------------
      // COMPUTE H-IPPS-PAY-AMT ROUNDED =
      //       H-STAND-AMT-OPER-PMT + H-CAPI-PMT.
      calculationContext.setHoldIppsPayAmount(
          calculationContext
              .getHoldStandardAmountOperatingPayment()
              .add(calculationContext.getHoldCapitalPayment())
              .setScale(2, RoundingMode.HALF_UP));

      // *** -------------------------------------------------------
      // *** IPPS COMPARABLE PER DIEM PAYMENT
      // *** -------------------------------------------------------
      // COMPUTE H-IPPS-PER-DIEM ROUNDED =
      //       (H-IPPS-PAY-AMT / H-IPPS-DRG-ALOS) * H-LOS.
      calculationContext.setHoldIppsPerDiem(
          calculationContext
              .getHoldIppsPayAmount()
              .divide(calculationContext.getHoldIppsDrgALengthOfStay(), 3, RoundingMode.DOWN)
              .multiply(calculationContext.getHoldLengthOfStay())
              .setScale(2, RoundingMode.HALF_UP));

      // IF H-IPPS-PER-DIEM > H-IPPS-PAY-AMT
      if (BigDecimalUtils.isGreaterThan(
          calculationContext.getHoldIppsPerDiem(), calculationContext.getHoldIppsPayAmount())) {
        // MOVE H-IPPS-PAY-AMT TO H-IPPS-PER-DIEM
        calculationContext.setHoldIppsPerDiem(
            calculationContext.getHoldIppsPayAmount().setScale(2, RoundingMode.HALF_UP));
      }
    }
  }

  // *3) DETERMINE THE PROVIDER'S GEOGRAPHIC CLASSIFICATION
  // *-----------------------------------------------------
  // *4) CALCULATE OPERATING DSH AMOUNT BASED ON GEOGRAPHIC CLASS
  // *-----------------------------------------------------------
  private void calculateOperatingDshAmount(
      LtchPricerContext calculationContext, CbsaProviderType cbsaProviderType) {
    // *3) DETERMINE THE PROVIDER'S GEOGRAPHIC CLASSIFICATION
    // *-----------------------------------------------------
    switch (determineGeographicalClass(cbsaProviderType, calculationContext)) {
        // GEOGRAPHIC CLASS 2
        // ------------------
      case "TWO":
        calculationContext.setHoldOperatingDshAmount(getOperatingDshAmount(calculationContext));
        break;
        // GEOGRAPHIC CLASS 3
        // ------------------
      case "THREE":
        calculationContext.setHoldOperatingDshAmount(getOperatingDshAmount(calculationContext));
        // IF H-OPER-DSH > .12
        //   MOVE .12 TO H-OPER-DSH
        // END-IF
        if (BigDecimalUtils.isGreaterThan(
            calculationContext.getHoldOperatingDshAmount(), new BigDecimal("0.12"))) {
          calculationContext.setHoldOperatingDshAmount(
              new BigDecimal("0.12").setScale(4, RoundingMode.HALF_UP));
        }
        break;
        // GEOGRAPHIC CLASS 4
        // ------------------
      case "FOUR":
        calculationContext.setHoldOperatingDshAmount(
            BigDecimal.ZERO.setScale(4, RoundingMode.HALF_UP));
        break;
    }

    // *** -------------------------------------------------------
    // *** CURRENT OPERATING DSH PAYMENT REDUCTION
    // *** -------------------------------------------------------
    // COMPUTE H-OPER-DSH ROUNDED =
    //             H-OPER-DSH * H-OPER-DSH-REDUCTION-FACTOR.
    calculationContext.setHoldOperatingDshAmount(
        calculationContext
            .getHoldOperatingDshAmount()
            .multiply(calculationContext.getHoldOperatingDshReductionFactor())
            .setScale(4, RoundingMode.HALF_UP));
  }

  private CbsaProviderType getCbsaProviderType(LtchPricerContext calculationContext) {
    CbsaProviderType cbsaProviderType;
    if ("R".equals(calculationContext.getHoldProvIppsCBSA().getSize())) {
      cbsaProviderType = CbsaProviderType.RURAL;
    } else {
      cbsaProviderType = CbsaProviderType.URBAN;
    }
    return cbsaProviderType;
  }

  private BigDecimal getOperatingDshAmount(LtchPricerContext calculationContext) {
    BigDecimal operatingDshAmount;
    BigDecimal operatingMin = new BigDecimal("0.15");
    BigDecimal operatingMax = new BigDecimal("0.202");
    // IF (H-OPER-DSH-PCT >= .15 AND <= .202)
    if (BigDecimalUtils.isGreaterThanOrEqualTo(
            calculationContext.getHoldOperatingDshPercent(), operatingMin)
        && BigDecimalUtils.isLessThanOrEqualTo(
            calculationContext.getHoldOperatingDshPercent(), operatingMax)) {
      // COMPUTE H-OPER-DSH ROUNDED =
      //               ((H-OPER-DSH-PCT - .15) * .65) + .025
      operatingDshAmount =
          calculationContext
              .getHoldOperatingDshPercent()
              .subtract(operatingMin)
              .multiply(new BigDecimal("0.65"))
              .add(new BigDecimal("0.025"));
    } else if (BigDecimalUtils.isGreaterThan(
        calculationContext.getHoldOperatingDshPercent(), operatingMax)) {
      // COMPUTE H-OPER-DSH ROUNDED =
      //                ((H-OPER-DSH-PCT - .202) * .825) + .0588
      operatingDshAmount =
          calculationContext
              .getHoldOperatingDshPercent()
              .subtract(operatingMax)
              .multiply(new BigDecimal("0.825"))
              .add(new BigDecimal("0.0588"));
    } else {
      operatingDshAmount = BigDecimal.ZERO;
    }
    return operatingDshAmount.setScale(4, RoundingMode.HALF_UP);
  }

  // *3) DETERMINE THE PROVIDER'S GEOGRAPHIC CLASSIFICATION
  // *-----------------------------------------------------
  private String determineGeographicalClass(
      CbsaProviderType cbsaProviderType, LtchPricerContext calculationContext) {
    // MOVE '4' TO H-GEO-CLASS (default)
    String geographicalClass = "FOUR";
    BigDecimal minimumOperatingDshPercentage = new BigDecimal("0.15");
    BigDecimal urbanBeds = new BigDecimal(URBAN_BEDS);
    BigDecimal ruralBeds = new BigDecimal(RURAL_BEDS);
    // H-OPER-DSH-PCT >= .15
    if (BigDecimalUtils.isGreaterThanOrEqualTo(
        calculationContext.getHoldOperatingDshPercent(), minimumOperatingDshPercentage)) {
      // URBAN, < 100 BEDS
      // RURAL, < 500 BEDS
      // IF URBAN-CBSA AND H-BED-SIZE < 100
      // IF RURAL-CBSA AND H-BED-SIZE < 500
      if (cbsaProviderType.equals(CbsaProviderType.URBAN)
              && BigDecimalUtils.isLessThan(calculationContext.getHoldBedSize(), urbanBeds)
          || cbsaProviderType.equals(CbsaProviderType.RURAL)
              && BigDecimalUtils.isLessThan(calculationContext.getHoldBedSize(), ruralBeds)) {
        geographicalClass = "THREE";
      }
      // URBAN, >= 100 BEDS
      // RURAL, >= 500 BEDS
      // IF URBAN-CBSA AND H-BED-SIZE >= 100
      // IF RURAL-CBSA AND H-BED-SIZE < 500
      else if (cbsaProviderType.equals(CbsaProviderType.URBAN)
              && BigDecimalUtils.isGreaterThanOrEqualTo(
                  calculationContext.getHoldBedSize(), urbanBeds)
          || cbsaProviderType.equals(CbsaProviderType.RURAL)
              && BigDecimalUtils.isGreaterThanOrEqualTo(
                  calculationContext.getHoldBedSize(), ruralBeds)) {
        geographicalClass = "TWO";
      }
    }
    return geographicalClass;
  }
}
