package gov.cms.fiss.pricers.ltch.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.ltch.api.BillingRecord;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import gov.cms.fiss.pricers.ltch.core.codes.ErrorCode;
import gov.cms.fiss.pricers.ltch.core.codes.PaymentType;
import gov.cms.fiss.pricers.ltch.core.codes.SecondaryPaymentTypeSiteNeutral;
import gov.cms.fiss.pricers.ltch.core.codes.SecondaryPaymentTypeStandard;

public class SetNewReturnCodes
    implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {

  @Override
  public void priceClaim(LtchPricerContext calculationContext) {
    ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();
    if (!ErrorCode.isErrorCode(prospectivePaymentRecord.getRtc())
        && calculationContext.isPaymentStandardNewSiteNeutralOrBlendTypes()) {
      setNewReturnCodes(calculationContext);
    }
  }

  /** 7200-SET-NEW-RETURN-CODES. SET RETURN CODES FOR NEW POLICY CLAIMS */
  private void setNewReturnCodes(LtchPricerContext calculationContext) {
    ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();
    BillingRecord billingRecord = calculationContext.getBillingRecord();

    // DEFAULT (NO PSYCH/REHAB NOR VENTILATOR SERVICE)
    // *--------------------------------------------------*
    // * VENTILATOR SERVICE PRESENT                       *
    // *--------------------------------------------------*
    prospectivePaymentRecord.setRtc1("C");
    boolean ventilatorServicePresent =
        billingRecord.getProcedureCodes().contains(calculationContext.getVentilationIcd10Code());
    if (ventilatorServicePresent) {
      prospectivePaymentRecord.setRtc1("B");
    }

    // *--------------------------------------------------*
    // * SITE NEUTRAL PMT BECAUSE PSYCH/REHAB DRG PRESENT *
    // * (PRESENCE OF PSCHY/REHAB DRG TRUMPS VENT PROC.)  *
    // *--------------------------------------------------*
    if (prospectivePaymentRecord.isPsychRehabDrg()
        && calculationContext.isPaymentBlendOrSiteNeutral()) {
      prospectivePaymentRecord.setRtc1("A");
    }

    // ***************************************************************
    // * SET THE SECOND POSITION OF RETURN CODE                      *
    // ***************************************************************
    setNewReturnCodesBlendAndSiteNeutralCost(calculationContext);
    setNewReturnCodesBlendAndSiteNeutralIpps(calculationContext);
    setNewReturnCodesSiteNeutral(calculationContext);
    setNewReturnCodesStandardNew(calculationContext);
  }

  private void setNewReturnCodesBlendAndSiteNeutralCost(LtchPricerContext calculationContext) {
    // *-------------------------------------------------------------*
    // * BLENDED PAYMENT CLAIMS; SITE NEUTRAL PORTION = COST         *
    // *-------------------------------------------------------------*
    PaymentType paymentType = calculationContext.getPaymentType();
    SecondaryPaymentTypeSiteNeutral secondaryPaymentTypeSiteNeutral =
        calculationContext.getSecondaryPaymentTypeSiteNeutral();
    ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();
    SecondaryPaymentTypeStandard secondaryPaymentTypeStandard =
        calculationContext.getSecondaryPaymentTypeStandard();

    if (paymentType.equals(PaymentType.BLEND)
        && secondaryPaymentTypeSiteNeutral.equals(SecondaryPaymentTypeSiteNeutral.COST)) {
      if (BigDecimalUtils.isZero(prospectivePaymentRecord.getOutlierPaymentAmount())
          && secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.FULL)) {
        prospectivePaymentRecord.setRtc2("0");
      }
      if (BigDecimalUtils.isGreaterThanZero(prospectivePaymentRecord.getOutlierPaymentAmount())
          && secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.FULL)) {
        prospectivePaymentRecord.setRtc2("1");
      }
      if (BigDecimalUtils.isZero(prospectivePaymentRecord.getOutlierPaymentAmount())
          && secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.SSO)) {
        prospectivePaymentRecord.setRtc2("2");
      }
      if (BigDecimalUtils.isGreaterThanZero(prospectivePaymentRecord.getOutlierPaymentAmount())
          && secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.SSO)) {
        prospectivePaymentRecord.setRtc2("3");
      }
    }
  }

  private void setNewReturnCodesBlendAndSiteNeutralIpps(LtchPricerContext calculationContext) {
    // *-------------------------------------------------------------*
    // * BLENDED PAYMENT CLAIMS; SITE NEUTRAL PORTION = IPPS COMP.   *
    // *-------------------------------------------------------------*
    PaymentType paymentType = calculationContext.getPaymentType();
    SecondaryPaymentTypeSiteNeutral secondaryPaymentTypeSiteNeutral =
        calculationContext.getSecondaryPaymentTypeSiteNeutral();
    ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();
    SecondaryPaymentTypeStandard secondaryPaymentTypeStandard =
        calculationContext.getSecondaryPaymentTypeStandard();

    if (paymentType.equals(PaymentType.BLEND)
        && secondaryPaymentTypeSiteNeutral.equals(SecondaryPaymentTypeSiteNeutral.IPPS)) {
      if (BigDecimalUtils.isZero(prospectivePaymentRecord.getOutlierPaymentAmount())
          && secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.FULL)) {
        prospectivePaymentRecord.setRtc2("4");
      }
      if (BigDecimalUtils.isGreaterThanZero(prospectivePaymentRecord.getOutlierPaymentAmount())
          && secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.FULL)) {
        prospectivePaymentRecord.setRtc2("5");
      }
      if (BigDecimalUtils.isZero(prospectivePaymentRecord.getOutlierPaymentAmount())
          && secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.SSO)) {
        prospectivePaymentRecord.setRtc2("6");
      }
      if (BigDecimalUtils.isGreaterThanZero(prospectivePaymentRecord.getOutlierPaymentAmount())
          && secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.SSO)) {
        prospectivePaymentRecord.setRtc2("7");
      }
    }
  }

  private void setNewReturnCodesSiteNeutral(LtchPricerContext calculationContext) {
    // *-------------------------------------------------------------*
    // * 100% SITE NEUTRAL PAYMENT CLAIMS                            *
    // *-------------------------------------------------------------*
    PaymentType paymentType = calculationContext.getPaymentType();
    SecondaryPaymentTypeSiteNeutral secondaryPaymentTypeSiteNeutral =
        calculationContext.getSecondaryPaymentTypeSiteNeutral();
    ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();

    if (paymentType.equals(PaymentType.SITE_NEUTRAL)) {
      if (secondaryPaymentTypeSiteNeutral.equals(SecondaryPaymentTypeSiteNeutral.COST)) {
        prospectivePaymentRecord.setRtc2("A");
      }
      if (secondaryPaymentTypeSiteNeutral.equals(SecondaryPaymentTypeSiteNeutral.IPPS)) {
        if (BigDecimalUtils.isZero(prospectivePaymentRecord.getOutlierPaymentAmount())) {
          prospectivePaymentRecord.setRtc2("B");
        } else {
          prospectivePaymentRecord.setRtc2("C");
        }
      }
    }
  }

  private void setNewReturnCodesStandardNew(LtchPricerContext calculationContext) {
    // *-------------------------------------------------------------*
    // * 100% STANDARD PAYMENT CLAIMS                                *
    // *-------------------------------------------------------------*
    PaymentType paymentType = calculationContext.getPaymentType();
    ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();
    SecondaryPaymentTypeStandard secondaryPaymentTypeStandard =
        calculationContext.getSecondaryPaymentTypeStandard();

    if (paymentType.equals(PaymentType.STANDARD_NEW)) {
      if (BigDecimalUtils.isZero(prospectivePaymentRecord.getOutlierPaymentAmount())
          && secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.SSO)) {
        prospectivePaymentRecord.setRtc2("D");
      }
      if (BigDecimalUtils.isGreaterThanZero(prospectivePaymentRecord.getOutlierPaymentAmount())
          && secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.SSO)) {
        prospectivePaymentRecord.setRtc2("E");
      }
      if (BigDecimalUtils.isZero(prospectivePaymentRecord.getOutlierPaymentAmount())
          && secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.FULL)) {
        prospectivePaymentRecord.setRtc2("F");
      }
      if (BigDecimalUtils.isGreaterThanZero(prospectivePaymentRecord.getOutlierPaymentAmount())
          && secondaryPaymentTypeStandard.equals(SecondaryPaymentTypeStandard.FULL)) {
        prospectivePaymentRecord.setRtc2("G");
      }
    }
  }
}
