package gov.cms.fiss.pricers.ltch.core.rules.rules2020;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.common.util.LocalDateUtils;
import gov.cms.fiss.pricers.ltch.api.BillingRecord;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.api.ProviderRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import gov.cms.fiss.pricers.ltch.core.tables.DataTables;
import gov.cms.fiss.pricers.ltch.core.tables.IppsCbsaTableEntry;
import gov.cms.fiss.pricers.ltch.core.tables.WageIppsIndexRuralEntry;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.Optional;
import org.apache.commons.lang3.StringUtils;

public class DetermineIppsWageIndex
    implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {

  /** Converted from 0575-GET-IPPS-CBSA. in COBOL. Sets the IPPS CBSA values for calculation. */
  @Override
  public void priceClaim(LtchPricerContext ltchPricerContext) {
    //    SET IPPS CBSA TO GEOGRAPHIC LOCATION CBSA IN PSF
    //    ADDED FOLLOWING LINE TO ALLOW LTCH PRICER TO WORK
    //    WITH THE NEW ARIZONA STATE CODE OF 00
    final ProviderRecord providerRecord = ltchPricerContext.getProviderRecord();
    final BillingRecord billingRecord = ltchPricerContext.getBillingRecord();
    final DataTables dataTables = ltchPricerContext.getDataTables();
    final ProspectivePaymentRecord prospectivePaymentRecord =
        ltchPricerContext.getProspectivePaymentRecord();
    String trimmedGeolocation =
        StringUtils.trimToEmpty(providerRecord.getCbsaData().getGeolocation());
    trimmedGeolocation = "00".equalsIgnoreCase(trimmedGeolocation) ? "03" : trimmedGeolocation;

    // lookup here
    IppsCbsaTableEntry entry =
        dataTables.getIppsCbsaWageIndex(trimmedGeolocation, providerRecord.getEffectiveDate());
    if (entry != null
        && billingRecord.getDischargeDate().isAfter(entry.getEffectiveDate())
        && ("98".equalsIgnoreCase(trimmedGeolocation)
            || "99".equalsIgnoreCase(trimmedGeolocation)
            || LocalDateUtils.inRange(
                entry.getEffectiveDate(),
                ltchPricerContext.getFyBegin(),
                ltchPricerContext.getFyEnd()))) {
      ltchPricerContext.setHoldProvIppsCBSA(entry);
    } else if (entry == null) {
      // set return code to 60
      prospectivePaymentRecord.setRtc("60");
    }
    // Set wage index floors
    assignFloorCBSA(
        ltchPricerContext,
        prospectivePaymentRecord,
        "00".equals(ltchPricerContext.getProviderRecord().getNewStateCode())
            ? "03"
            : ltchPricerContext.getProviderRecord().getNewStateCode());

    if (entry != null
        && BigDecimalUtils.isGreaterThan(
            ltchPricerContext.getWageIndexFloor().getWageIndex(), entry.getWageIndex())) {
      IppsCbsaTableEntry temp = new IppsCbsaTableEntry();
      temp.setCbsa(
          ltchPricerContext.getWageIndexFloor().getCbsa123()
              + ltchPricerContext.getWageIndexFloor().getCbsa45());
      temp.setEffectiveDate(ltchPricerContext.getWageIndexFloor().getEffectiveDate());
      temp.setWageIndex(ltchPricerContext.getWageIndexFloor().getWageIndex());
      entry = temp;
    }

    // Get the size indicator
    if (entry != null && billingRecord.getDischargeDate().isAfter(entry.getEffectiveDate())) {
      // COBOL checks to confirm that the first 3 chars of the geolocation string are spaces
      if (trimmedGeolocation.length() <= 2) {
        entry.setSize("R");
      } else if ("L".equals(entry.getSize())) {
        entry.setSize("L");
      } else {
        entry.setSize("O");
      }
    }
    setIppsHolderVariables(ltchPricerContext, entry);
  }

  private void setIppsHolderVariables(
      LtchPricerContext ltchPricerContext, IppsCbsaTableEntry entry) {
    ltchPricerContext.setHoldProvIppsCBSA(entry);
    if (entry != null && BigDecimalUtils.isGreaterThan(entry.getWageIndex(), BigDecimal.ONE)) {
      ltchPricerContext.setHoldIppsNationalLaborShare(
          ltchPricerContext.getHighIppsNationalLaborShare());
      ltchPricerContext.setHoldIppsNationalNonLaborShare(
          ltchPricerContext.getHighIppsNationalNonLaborShare());
    } else {
      ltchPricerContext.setHoldIppsNationalLaborShare(
          ltchPricerContext.getLowIppsNationalLaborShare());
      ltchPricerContext.setHoldIppsNationalNonLaborShare(
          ltchPricerContext.getLowIppsNationalNonLaborShare());
    }
  }

  /**
   * Converted from 0580-FY2015-LATER-FLOOR-CBSA in COBOL. Gets the wage index floor values for
   * calculation.
   */
  private void assignFloorCBSA(
      LtchPricerContext ltchPricerContext,
      ProspectivePaymentRecord prospectivePaymentRecord,
      String geolocation) {
    WageIppsIndexRuralEntry ruralEntry = new WageIppsIndexRuralEntry();
    LocalDate dischargeDate = ltchPricerContext.getBillingRecord().getDischargeDate();

    if ("00".equals(prospectivePaymentRecord.getRtc())) {
      ruralEntry = getRuralFloorIpps(ltchPricerContext, geolocation, dischargeDate);
    }
    ltchPricerContext.setWageIndexFloor(ruralEntry);
  }

  /** Converted from 0190-GET-RURAL-FLOOR-IPPS in COBOL. */
  private WageIppsIndexRuralEntry getRuralFloorIpps(
      LtchPricerContext context, String geolocation, LocalDate dischargeDate) {
    return Optional.ofNullable(
            context.getDataTables().getWageIppsIndexRural(geolocation, dischargeDate))
        .orElse(new WageIppsIndexRuralEntry());
  }
}
