package gov.cms.fiss.pricers.ltch.core.rules.rules2021;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.common.util.LocalDateUtils;
import gov.cms.fiss.pricers.ltch.api.BillingRecord;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.api.ProviderRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import gov.cms.fiss.pricers.ltch.core.tables.DataTables;
import gov.cms.fiss.pricers.ltch.core.tables.IppsCbsaTableEntry;
import java.math.BigDecimal;
import java.time.LocalDate;

public class DetermineIppsWageIndex2021
    implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {

  /** Converted from 0575-GET-IPPS-CBSA. in COBOL. Sets the IPPS CBSA values for calculation. */
  @Override
  public void priceClaim(LtchPricerContext ltchPricerContext) {
    //    SET IPPS CBSA TO GEOGRAPHIC LOCATION CBSA IN PSF
    //    ADDED FOLLOWING LINE TO ALLOW LTCH PRICER TO WORK
    //    WITH THE NEW ARIZONA STATE CODE OF 00
    ProviderRecord providerRecord = ltchPricerContext.getProviderRecord();
    BillingRecord billingRecord = ltchPricerContext.getBillingRecord();
    DataTables dataTables = ltchPricerContext.getDataTables();
    ProspectivePaymentRecord prospectivePaymentRecord =
        ltchPricerContext.getProspectivePaymentRecord();
    String trimmedGeolocation =
        providerRecord.getCbsaData().getGeolocation() != null
            ? providerRecord.getCbsaData().getGeolocation().trim()
            : "";
    trimmedGeolocation = "00".equalsIgnoreCase(trimmedGeolocation) ? "03" : trimmedGeolocation;

    // lookup here
    IppsCbsaTableEntry entry =
        dataTables.getIppsCbsaWageIndex(trimmedGeolocation, providerRecord.getEffectiveDate());
    if (entry != null
        && billingRecord.getDischargeDate().isAfter(entry.getEffectiveDate())
        && ("98".equalsIgnoreCase(trimmedGeolocation)
            || "99".equalsIgnoreCase(trimmedGeolocation)
            || LocalDateUtils.inRange(
                entry.getEffectiveDate(),
                ltchPricerContext.getFyBegin(),
                ltchPricerContext.getFyEnd()))) {
      ltchPricerContext.setHoldProvIppsCBSA(entry);
    } else if (entry == null) {
      // set return code to 60
      prospectivePaymentRecord.setRtc("60");
    }
    // Set wage index floors
    assignFloorCBSA(
        ltchPricerContext,
        prospectivePaymentRecord,
        "00".equals(ltchPricerContext.getProviderRecord().getNewStateCode())
            ? "03"
            : ltchPricerContext.getProviderRecord().getNewStateCode());

    if (entry != null
        && BigDecimalUtils.isGreaterThan(
            ltchPricerContext.getRuralWageIndexFloor().getRuralWageIndex(), entry.getWageIndex())) {
      IppsCbsaTableEntry temp = new IppsCbsaTableEntry();
      temp.setCbsa(ltchPricerContext.getRuralWageIndexFloor().getCbsa());
      temp.setEffectiveDate(ltchPricerContext.getRuralWageIndexFloor().getEffectiveDate());
      temp.setWageIndex(ltchPricerContext.getRuralWageIndexFloor().getRuralWageIndex());
      entry = temp;
    }

    // Get the size indicator
    if (entry != null && billingRecord.getDischargeDate().isAfter(entry.getEffectiveDate())) {
      // COBOL checks to confirm that the first 3 chars of the geolocation string are spaces
      if (trimmedGeolocation.length() <= 2) {
        entry.setSize("R");
      } else if ("L".equals(entry.getSize())) {
        entry.setSize("L");
      } else {
        entry.setSize("O");
      }
    }
    setIppsHolderVariables(ltchPricerContext, entry);
  }

  private void setIppsHolderVariables(
      LtchPricerContext ltchPricerContext, IppsCbsaTableEntry entry) {
    ltchPricerContext.setHoldProvIppsCBSA(entry);
    if (entry != null && BigDecimalUtils.isGreaterThan(entry.getWageIndex(), BigDecimal.ONE)) {
      ltchPricerContext.setHoldIppsNationalLaborShare(
          ltchPricerContext.getHighIppsNationalLaborShare());
      ltchPricerContext.setHoldIppsNationalNonLaborShare(
          ltchPricerContext.getHighIppsNationalNonLaborShare());
    } else {
      ltchPricerContext.setHoldIppsNationalLaborShare(
          ltchPricerContext.getLowIppsNationalLaborShare());
      ltchPricerContext.setHoldIppsNationalNonLaborShare(
          ltchPricerContext.getLowIppsNationalNonLaborShare());
    }
  }

  /**
   * Converted from 0580-FY2015-LATER-FLOOR-CBSA in COBOL. Gets the wage index floor values for
   * calculation.
   */
  private void assignFloorCBSA(
      LtchPricerContext ltchPricerContext,
      ProspectivePaymentRecord prospectivePaymentRecord,
      String geolocation) {
    LocalDate dischargeDate = ltchPricerContext.getBillingRecord().getDischargeDate();
    IppsCbsaTableEntry entry = new IppsCbsaTableEntry();
    if ("00".equals(prospectivePaymentRecord.getRtc())) {
      entry = getRuralFloorIpps(ltchPricerContext, geolocation, dischargeDate);
    }
    ltchPricerContext.setRuralWageIndexFloor(entry);
  }

  /** Converted from 0190-GET-RURAL-FLOOR-IPPS in COBOL. */
  private IppsCbsaTableEntry getRuralFloorIpps(
      LtchPricerContext context, String geolocation, LocalDate dischargeDate) {
    IppsCbsaTableEntry ruralEntry =
        context.getDataTables().getIppsCbsaWageIndex(geolocation, dischargeDate);
    if (ruralEntry == null) {
      ruralEntry = new IppsCbsaTableEntry();
      ruralEntry.setCbsa("00");
      ruralEntry.setEffectiveDate(null);
      ruralEntry.setWageIndex(BigDecimal.ZERO);
    }
    return ruralEntry;
  }
}
