package gov.cms.fiss.pricers.ltch.core.rules.rules2022;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.common.util.LocalDateUtils;
import gov.cms.fiss.pricers.ltch.api.BillingRecord;
import gov.cms.fiss.pricers.ltch.api.LtchInput;
import gov.cms.fiss.pricers.ltch.api.LtchOutput;
import gov.cms.fiss.pricers.ltch.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.ltch.api.ProviderRecord;
import gov.cms.fiss.pricers.ltch.core.LtchPricerContext;
import gov.cms.fiss.pricers.ltch.core.tables.DataTables;
import gov.cms.fiss.pricers.ltch.core.tables.IppsCbsaTableEntry;
import java.math.BigDecimal;
import java.time.LocalDate;

public class DetermineIppsWageIndex2022
    implements CalculationRule<LtchInput, LtchOutput, LtchPricerContext> {

  /** Converted from 0575-GET-IPPS-CBSA. in COBOL. Sets the IPPS CBSA values for calculation. */
  @Override
  public void priceClaim(LtchPricerContext ltchPricerContext) {
    //    SET IPPS CBSA TO GEOGRAPHIC LOCATION CBSA IN PSF
    //    ADDED FOLLOWING LINE TO ALLOW LTCH PRICER TO WORK
    //    WITH THE NEW ARIZONA STATE CODE OF 00
    final ProviderRecord providerRecord = ltchPricerContext.getProviderRecord();
    final BillingRecord billingRecord = ltchPricerContext.getBillingRecord();
    final DataTables dataTables = ltchPricerContext.getDataTables();
    final ProspectivePaymentRecord prospectivePaymentRecord =
        ltchPricerContext.getProspectivePaymentRecord();
    String trimmedGeolocation =
        providerRecord.getCbsaData().getGeolocation() != null
            ? providerRecord.getCbsaData().getGeolocation().trim()
            : "";
    trimmedGeolocation = "00".equalsIgnoreCase(trimmedGeolocation) ? "03" : trimmedGeolocation;
    // lookup here
    IppsCbsaTableEntry entry =
        dataTables.getIppsCbsaWageIndex(trimmedGeolocation, providerRecord.getEffectiveDate());
    if (entry != null
        && billingRecord.getDischargeDate().isAfter(entry.getEffectiveDate())
        && ("98".equalsIgnoreCase(trimmedGeolocation)
            || "99".equalsIgnoreCase(trimmedGeolocation)
            || LocalDateUtils.inRange(
                entry.getEffectiveDate(),
                ltchPricerContext.getFyBegin(),
                ltchPricerContext.getFyEnd()))) {
      ltchPricerContext.setHoldProvIppsCBSA(entry);
    } else if (entry == null) {
      // set return code to 60
      prospectivePaymentRecord.setRtc("60");
    }
    // Set wage index floors
    assignFloorCBSA(
        ltchPricerContext,
        prospectivePaymentRecord,
        "00".equals(ltchPricerContext.getProviderRecord().getNewStateCode())
            ? "03"
            : ltchPricerContext.getProviderRecord().getNewStateCode());

    if (entry != null
        && BigDecimalUtils.isGreaterThan(
            ltchPricerContext.getRuralWageIndexFloor().getRuralWageIndex(), entry.getWageIndex())) {
      final IppsCbsaTableEntry temp = new IppsCbsaTableEntry();
      temp.setCbsa(ltchPricerContext.getRuralWageIndexFloor().getCbsa());
      temp.setEffectiveDate(ltchPricerContext.getRuralWageIndexFloor().getEffectiveDate());
      temp.setWageIndex(ltchPricerContext.getRuralWageIndexFloor().getRuralWageIndex());
      entry = temp;
    }

    /*
     * For the IPPS wage index, the imputed wage floor policy is being reintroduced and made
     * permanent effective FY 2022. As is being done in the IPPS Pricer, the LTCH Pricer will
     * need to apply the imputed wage floor after the rural floor has been applied when
     * determining the IPPS wage index.
     * This adjustment only applies to a handful of states.
     * August 2021 - The following specs were provided for implementing the imputed wage floor for
     * the IPPS Pricer.
     * After the rural floor has been applied, the Pricer will need to do the following:
     * 1.	Use provider’s state code to pull value from fourth column of wage index table.
     * 2.	If fourth column value in wage index table is null then do nothing.
     * 3.	If fourth column value in wage index table is not null do the following:
     *    a.	If value in fourth column is greater than the providers wage index determined up to
     *  this point, then set providers wage index equal to fourth column value. If value in
     * fourth column is less than or equal to the providers wage index determined up to this
     * point, then do nothing.
     */
    ltchPricerContext.setImputedWageIndexFloor(
        getImputedWageFloorIpps(
            ltchPricerContext,
            "00".equals(ltchPricerContext.getProviderRecord().getNewStateCode())
                ? "03"
                : ltchPricerContext.getProviderRecord().getNewStateCode(),
            billingRecord.getDischargeDate()));
    if (entry != null
        && BigDecimalUtils.isGreaterThan(
            ltchPricerContext.getImputedWageIndexFloor().getImputedWageIndex(),
            entry.getWageIndex())) {
      final IppsCbsaTableEntry temp = new IppsCbsaTableEntry();
      temp.setCbsa(ltchPricerContext.getImputedWageIndexFloor().getCbsa());
      temp.setEffectiveDate(ltchPricerContext.getImputedWageIndexFloor().getEffectiveDate());
      temp.setWageIndex(ltchPricerContext.getImputedWageIndexFloor().getImputedWageIndex());
      entry = temp;
    }

    setIppsCbsaTableEntrySize(billingRecord, trimmedGeolocation, entry);
    setIppsHolderVariables(ltchPricerContext, entry);
  }

  private void setIppsCbsaTableEntrySize(
      BillingRecord billingRecord, String trimmedGeolocation, IppsCbsaTableEntry entry) {
    // Get the size indicator
    if (entry != null && billingRecord.getDischargeDate().isAfter(entry.getEffectiveDate())) {
      // COBOL checks to confirm that the first 3 chars of the geolocation string are spaces
      if (trimmedGeolocation.length() <= 2) {
        entry.setSize("R");
      } else if ("L".equals(entry.getSize())) {
        entry.setSize("L");
      } else {
        entry.setSize("O");
      }
    }
  }

  private void setIppsHolderVariables(
      LtchPricerContext ltchPricerContext, IppsCbsaTableEntry entry) {
    ltchPricerContext.setHoldProvIppsCBSA(entry);
    if (entry != null && BigDecimalUtils.isGreaterThan(entry.getWageIndex(), BigDecimal.ONE)) {
      ltchPricerContext.setHoldIppsNationalLaborShare(
          ltchPricerContext.getHighIppsNationalLaborShare());
      ltchPricerContext.setHoldIppsNationalNonLaborShare(
          ltchPricerContext.getHighIppsNationalNonLaborShare());
    } else {
      ltchPricerContext.setHoldIppsNationalLaborShare(
          ltchPricerContext.getLowIppsNationalLaborShare());
      ltchPricerContext.setHoldIppsNationalNonLaborShare(
          ltchPricerContext.getLowIppsNationalNonLaborShare());
    }
  }

  /**
   * Converted from 0580-FY2015-LATER-FLOOR-CBSA in COBOL. Gets the wage index floor values for
   * calculation.
   */
  private void assignFloorCBSA(
      LtchPricerContext ltchPricerContext,
      ProspectivePaymentRecord prospectivePaymentRecord,
      String geolocation) {
    final LocalDate dischargeDate = ltchPricerContext.getBillingRecord().getDischargeDate();
    IppsCbsaTableEntry entry = new IppsCbsaTableEntry();
    if ("00".equals(prospectivePaymentRecord.getRtc())) {
      entry = getRuralFloorIpps(ltchPricerContext, geolocation, dischargeDate);
    }
    ltchPricerContext.setRuralWageIndexFloor(entry);
  }

  /** Converted from 0190-GET-RURAL-FLOOR-IPPS in COBOL. */
  private IppsCbsaTableEntry getRuralFloorIpps(
      LtchPricerContext context, String geolocation, LocalDate dischargeDate) {
    IppsCbsaTableEntry ruralEntry =
        context.getDataTables().getIppsCbsaWageIndex(geolocation, dischargeDate);
    if (ruralEntry == null) {
      ruralEntry = new IppsCbsaTableEntry();
      ruralEntry.setCbsa("00");
      ruralEntry.setEffectiveDate(null);
      ruralEntry.setWageIndex(BigDecimal.ZERO);
    }
    return ruralEntry;
  }

  private IppsCbsaTableEntry getImputedWageFloorIpps(
      LtchPricerContext context, String geolocation, LocalDate dischargeDate) {
    IppsCbsaTableEntry imputedEntry =
        context.getDataTables().getIppsCbsaWageIndex(geolocation, dischargeDate);
    if (imputedEntry == null) {
      imputedEntry = new IppsCbsaTableEntry();
      imputedEntry.setCbsa("00");
      imputedEntry.setEffectiveDate(null);
      imputedEntry.setWageIndex(BigDecimal.ZERO);
    }
    return imputedEntry;
  }
}
